import unittest
from unittest import mock
from flask import Flask
from markupsafe import escape
from .. import links


def mock_url_for(endpoint, **kwargs):
    if endpoint == "abs_by_id":
        paper_id = kwargs["paper_id"]
        return f"https://arxiv.org/abs/{paper_id}"


class Id_Patterns_Test(unittest.TestCase):
    def setUp(self):
        self.app = Flask("bogus")

    def test_arxiv_ids(self):
        def find_match(txt):
            with self.app.app_context():
                ptn = links._get_pattern(links.DEFAULT_KINDS)
                return ptn.search(txt)

        self.assertIsNotNone(find_match("math/9901123"))
        self.assertIsNotNone(find_match("hep-ex/9901123"))
        self.assertIsNotNone(find_match("gr-qc/9901123"))

        self.assertIsNotNone(find_match("1202.1234"))
        self.assertIsNotNone(find_match("1202.1234v1"))
        self.assertIsNotNone(find_match("1203.12345"))
        self.assertIsNotNone(find_match("1203.12345v1"))
        self.assertIsNotNone(find_match("1203.12345v12"))

        # slightly odd but seen in comments
        self.assertIsNotNone(find_match("hep-ph/1203.12345v12"))

    def test_find_match(self):
        def find_match(txt):
            with self.app.app_context():
                ptn = links._get_pattern(links.DEFAULT_KINDS)
                return ptn.search(txt)

        self.assertIsNone(find_match("junk"))
        self.assertIsNone(find_match(""))
        self.assertIsNone(find_match(" "))

        self.assertIsNotNone(find_match("doi:10.1002/0470841559.ch1"))
        self.assertIsNotNone(find_match("doi:10.1038/nphys1170"))

        self.assertIsNotNone(find_match("http://arxiv.org"))
        self.assertIsNotNone(find_match("http://arxiv.org?something=1"))
        self.assertIsNotNone(find_match("http://arxiv.org?something=1&anohter=2"))
        self.assertIsNotNone(find_match('"http://arxiv.org"'))


class TestURLize(unittest.TestCase):
    def setUp(self):
        self.app = Flask("bogus")

    def test_dont_urlize_cats(self):
        self.assertGreater(
            len(links.CATEGORIES_THAT_COULD_BE_HOSTNAMES),
            0,
            "The links.CATEGORIES_THAT_COULD_BE_HOSTNAMES must not be empty or it will match everything",
        )

    @mock.patch(f"{links.__name__}.clickthrough")
    def test_doi(self, mock_clickthrough):
        with self.app.app_context():
            mock_clickthrough.clickthrough_url = (
                lambda url: f"http://arxiv.org/clickthrough?url={url}"
            )
            self.maxDiff = 3000
            self.assertEqual(
                links.urlize(
                    "here is a rando doi doi:10.1109/5.771073 that needs a link"
                ),
                'here is a rando doi doi:<a href="https://doi.org/10.1109/5.771073" data-doi="10.1109/5.771073" class="link-https link-external" rel="external noopener nofollow">https://doi.org/10.1109/5.771073</a> that needs a link',
            )

    def test_transform_token(self):
        # def doi_id_url_transform_token(tkn,fn):
        #     return doi_id_url_transform_token(fn, tkn)
        with self.app.app_context():
            self.assertEqual(links.urlize("", ["url"]), "")

            self.assertEqual(
                links.urlize("it is fine, chapter 234 see<xxyx,234>"),
                escape("it is fine, chapter 234 see<xxyx,234>"),
            )

            self.assertEqual(
                links.urlize("http://arxiv.org", ["url"]),
                '<a href="http://arxiv.org" class="link-internal link-http">http://arxiv.org</a>',
            )

            self.assertEqual(
                links.urlize("https://arxiv.org", ["url"]),
                '<a href="https://arxiv.org" class="link-internal link-https">https://arxiv.org</a>',
            )

            self.assertEqual(
                links.urlize("in the front http://arxiv.org oth", ["url"]),
                'in the front <a href="http://arxiv.org" class="link-internal link-http">http://arxiv.org</a> oth',
            )

            self.assertEqual(
                links.urlize(".http://arxiv.org.", ["url"]),
                '.<a href="http://arxiv.org" class="link-internal link-http">http://arxiv.org</a>.',
            )

            self.assertEqual(
                links.urlize('"http://arxiv.org"', ["url"]),
                '"<a href="http://arxiv.org" class="link-internal link-http">http://arxiv.org</a>"',
            )

            self.assertEqual(
                links.urlize('"https://arxiv.org/help"', ["url"]),
                '"<a href="https://arxiv.org/help" class="link-internal link-https">https://arxiv.org/help</a>"',
            )

    @mock.patch(f"{links.__name__}.clickthrough")
    @mock.patch(f"{links.__name__}.url_for", mock_url_for)
    def test_urlize(self, mock_clickthrough):
        with self.app.app_context():
            mock_clickthrough.clickthrough_url.return_value = "foo"
            self.assertEqual(
                links.urlize("http://example.com/"),
                '<a href="http://example.com/" rel="external noopener nofollow" class="link-external link-http">this http URL</a>',
                "urlize (URL linking) 1/6",
            )
            self.assertEqual(
                links.urlize("https://example.com/"),
                '<a href="https://example.com/" rel="external noopener nofollow" class="link-external link-https">this https URL</a>',
                "urlize (URL linking) 2/6",
            )
            self.assertEqual(
                links.urlize("ftp://example.com/"),
                '<a href="ftp://example.com/" rel="external noopener nofollow" class="link-external link-ftp">this ftp URL</a>',
                "urlize (URL linking) 3/6",
            )

            self.assertEqual(
                links.urlize("http://projecteuclid.org/euclid.bj/1151525136"),
                '<a href="http://projecteuclid.org/euclid.bj/1151525136" rel="external noopener nofollow" class="link-external link-http">this http URL</a>',
                "urlize (URL linking) 6/6",
            )

            self.assertEqual(
                links.urlize("2448446.4710(5)"),
                "2448446.4710(5)",
                "urlize (should not match) 1/9",
            )
            self.assertEqual(
                links.urlize("HJD=2450274.4156+/-0.0009"),
                "HJD=2450274.4156+/-0.0009",
                "urlize (should not match) 2/9",
            )
            self.assertEqual(
                links.urlize("T_min[HJD]=49238.83662(14)+0.146352739(11)E."),
                "T_min[HJD]=49238.83662(14)+0.146352739(11)E.",
                "urlize (should not match) 3/9",
            )
            self.assertEqual(
                links.urlize("Pspin=1008.3408s"),
                "Pspin=1008.3408s",
                "urlize (should not match) 4/9",
            )
            self.assertEqual(
                links.urlize("2453527.87455^{+0.00085}_{-0.00091}"),
                "2453527.87455^{+0.00085}_{-0.00091}",
                "urlize (should not match) 5/9",
            )
            self.assertEqual(
                links.urlize("2451435.4353"),
                "2451435.4353",
                "urlize (should not match) 6/9",
            )

            self.assertEqual(
                links.urlize("cond-mat/97063007"),
                '<a href="https://arxiv.org/abs/cond-mat/9706300" data-arxiv-id="cond-mat/9706300" class="link-https">cond-mat/9706300</a>7',
                "urlize (should match) 7/9",
            )

            self.assertEqual(
                links.urlize("[http://onion.com/something-funny-about-arxiv-1234]"),
                '[<a href="http://onion.com/something-funny-about-arxiv-1234" rel="external noopener nofollow" class="link-external link-http">this http URL</a>]',
            )

            self.assertEqual(
                links.urlize("[http://onion.com/?q=something-funny-about-arxiv.1234]"),
                '[<a href="http://onion.com/?q=something-funny-about-arxiv.1234" rel="external noopener nofollow" class="link-external link-http">this http URL</a>]',
            )

            self.assertEqual(
                links.urlize("http://onion.com/?q=something funny"),
                '<a href="http://onion.com/?q=something" rel="external noopener nofollow" class="link-external link-http">this http URL</a> funny',
                "Spaces CANNOT be expected to be part of URLs",
            )

            self.assertEqual(
                links.urlize('"http://onion.com/something-funny-about-arxiv-1234"'),
                '"<a href="http://onion.com/something-funny-about-arxiv-1234" rel="external noopener nofollow" class="link-external link-http">this http URL</a>"',
                "Should handle URL surrounded by double quotes",
            )

            self.assertEqual(
                links.urlize("< http://example.com/1<2 ><"),
                '&lt; <a href="http://example.com/1" rel="external noopener nofollow" class="link-external link-http">this http URL</a>&lt;2 &gt;&lt;',
                "urlize (URL linking) 5/6",
            )

            self.assertEqual(
                links.urlize(
                    'Accepted for publication in A&A. The data will be available via CDS, and can be found "http://atlasgal.mpifr-bonn.mpg.de/cgi-bin/ATLASGAL_FILAMENTS.cgi"'
                ),
                'Accepted for publication in A&amp;A. The data will be available via CDS, and can be found "<a href="http://atlasgal.mpifr-bonn.mpg.de/cgi-bin/ATLASGAL_FILAMENTS.cgi" rel="external noopener nofollow" class="link-external link-http">this http URL</a>"',
            )

            self.assertEqual(
                links.urlize(
                    "see http://www.tandfonline.com/doi/abs/doi:10.1080/15980316.2013.860928?journalCode=tjid20"
                ),
                'see <a href="http://www.tandfonline.com/doi/abs/doi:10.1080/15980316.2013.860928?journalCode=tjid20" rel="external noopener nofollow" class="link-external link-http">this http URL</a>',
            )

            self.assertEqual(
                links.urlize("http://authors.elsevier.com/a/1TcSd,Ig45ZtO"),
                '<a href="http://authors.elsevier.com/a/1TcSd,Ig45ZtO" rel="external noopener nofollow" class="link-external link-http">this http URL</a>',
            )

        @mock.patch(f"{links.__name__}.url_for", mock_url_for)
        def test_category_id(self):
            self.assertEqual(
                links.urlize("version of arXiv.math.GR/0512484 (2011).", ["arxiv_id"]),
                'version of arXiv.<a data-arxiv-id="math.GR/0512484" href="https://arxiv.org/abs/math.GR/0512484" class="link-https">math.GR/0512484</a> (2011).',
            )

    @mock.patch(f"{links.__name__}.url_for", mock_url_for)
    def test_parens(self):
        """ARXIVNG-250 Linkification should not choke on parentheses."""
        with self.app.app_context():
            self.assertEqual(
                links.urlize(
                    "http://www-nuclear.univer.kharkov.ua/lib/1017_3(55)_12_p28-59.pdf"
                ),
                '<a href="http://www-nuclear.univer.kharkov.ua/lib/1017_3(55)_12_p28-59.pdf" rel="external noopener nofollow" class="link-external link-http">this http URL</a>',
            )

    @mock.patch(f"{links.__name__}.url_for", mock_url_for)
    def test_hosts(self):
        with self.app.app_context():
            self.assertEqual(
                links.urlize(
                    "can be downloaded from http://rwcc.bao.ac.cn:8001/swap/NLFFF_DBIE_code/HeHan_NLFFF_JGR.pdf"
                ),
                'can be downloaded from <a href="http://rwcc.bao.ac.cn:8001/swap/NLFFF_DBIE_code/HeHan_NLFFF_JGR.pdf" rel="external noopener nofollow" class="link-external link-http">this http URL</a>',
                "Should deal with ports correctly",
            )
            self.assertEqual(
                links.urlize(
                    "images is at http://85.20.11.14/hosting/punsly/APJLetter4.2.07/"
                ),
                'images is at <a href="http://85.20.11.14/hosting/punsly/APJLetter4.2.07/" rel="external noopener nofollow" class="link-external link-http">this http URL</a>',
                "should deal with numeric IP correctly",
            )

    @mock.patch(f"{links.__name__}.url_for", mock_url_for)
    def test_urls_with_plus(self):
        with self.app.app_context():
            self.assertEqual(
                links.urlize(
                    "http://www.fkf.mpg.de/andersen/docs/pub/abstract2004+/pavarini_02.pdf"
                ),
                '<a href="http://www.fkf.mpg.de/andersen/docs/pub/abstract2004+/pavarini_02.pdf" rel="external noopener nofollow" class="link-external link-http">this http URL</a>',
            )

    @mock.patch(f"{links.__name__}.url_for", mock_url_for)
    def test_anchors_with_slash(self):
        with self.app.app_context():
            self.assertIn(
                'href="https://dms.sztaki.hu/ecml-pkkd-2016/#/app/privateleaderboard"',
                links.urlize(
                    "https://dms.sztaki.hu/ecml-pkkd-2016/#/app/privateleaderboard"
                ),
                "Should deal with slash in URL anchor correctly",
            )

    @mock.patch(f"{links.__name__}.url_for", mock_url_for)
    def test_ftp(self):
        with self.app.app_context():
            self.assertEqual(
                links.urlize(
                    "7 Pages; ftp://ftp%40micrognu%2Ecom:anon%40anon@ftp.micrognu.com/pnenp/conclusion.pdf"
                ),
                '7 Pages; <a href="ftp://ftp%40micrognu%2Ecom:anon%40anon@ftp.micrognu.com/pnenp/conclusion.pdf" rel="external noopener nofollow" class="link-external link-ftp">this ftp URL</a>',
            )

    @mock.patch(f"{links.__name__}.url_for", mock_url_for)
    def test_arxiv_prefix(self):
        with self.app.app_context():
            urlized = links.urlize("see arxiv:1201.12345")
            self.assertIn('see <a ', urlized)
            self.assertIn('class="link-https"', urlized)
            self.assertIn('data-arxiv-id="1201.12345"', urlized)
            self.assertIn('href="https://arxiv.org/abs/1201.12345"', urlized)
            self.assertIn('>arXiv:1201.12345</a>', urlized)


    @mock.patch(f"{links.__name__}.clickthrough")
    def test_doi_2(self, mock_clickthrough):
        with self.app.app_context():
            mock_clickthrough.clickthrough_url = lambda x: x
            self.assertRegex(
                links.urlize("10.1088/1475-7516/2018/07/009"),
                r'<a.*href="https://.*10.1088/1475-7516/2018/07/009".*>https://doi.org/10.1088/1475-7516/2018/07/009</a>',
            )

            self.assertRegex(
                links.urlize("10.1088/1475-7516/2019/02/E02/meta"),
                r'<a.*href="https://.*10.1088/1475-7516/2019/02/E02/meta".*>https://doi.org/10.1088/1475-7516/2019/02/E02/meta</a>',
            )
            self.assertRegex(
                links.urlize("10.1088/1475-7516/2019/02/E02/META"),
                r'<a.*href="https://.*10.1088/1475-7516/2019/02/E02/META".*>https://doi.org/10.1088/1475-7516/2019/02/E02/META</a>',
            )
            self.assertRegex(
                links.urlize("doi:10.1088/1475-7516/2018/07/009"),
                r'<a.*href="https://.*10.1088/1475-7516/2018/07/009".*>https://doi.org/10.1088/1475-7516/2018/07/009</a>',
            )

            self.assertRegex(
                links.urlize("doi:10.1088/1475-7516/2019/02/E02/meta"),
                r'<a.*href="https://.*10.1088/1475-7516/2019/02/E02/meta".*>https://doi.org/10.1088/1475-7516/2019/02/E02/meta</a>',
            )
            self.assertRegex(
                links.urlize("doi:10.1088/1475-7516/2019/02/E02/META"),
                r'<a.*href="https://.*10.1088/1475-7516/2019/02/E02/META".*>https://doi.org/10.1088/1475-7516/2019/02/E02/META</a>',
            )

    @mock.patch(f"{links.__name__}.clickthrough")
    def test_double_doi(self, mock_clickthrough):
        with self.app.app_context():
            mock_clickthrough.clickthrough_url = lambda x: x
            txt = links.urlize(
                "10.1088/1475-7516/2018/07/009 10.1088/1475-7516/2019/02/E02/meta"
            )
            self.assertNotRegex(
                txt,
                r"this.*URL",
                'DOIs should not get the generic "this https URL" they should have the DOI text',
            )
            self.assertRegex(
                txt,
                r"<a.*>https://doi.org/10.1088/1475-7516/2018/07/009</a> <a.*>https://doi.org/10.1088/1475-7516/2019/02/E02/meta</a>",
                "Should handle two DOIs in a row correctly",
            )

    @mock.patch(f"{links.__name__}.clickthrough")
    def test_broad_doi(self, mock_clickthrough):
        with self.app.app_context():
            mock_clickthrough.clickthrough_url = lambda x: x

            broad_doi_fn = links.urlizer(["doi_field"])

            # ARXIVNG-3523 unusual DOI
            self.assertRegex(
                broad_doi_fn("21.11130/00-1735-0000-0005-146A-E"),
                r'<a.*href="https://.*21.11130.*>21.11130/00-1735-0000-0005-146A-E</a>',
            )

            self.assertRegex(
                broad_doi_fn("10.1088/1475-7516/2018/07/009"),
                r'<a.*href="https://.*10.1088/1475-7516/2018/07/009".*>https://doi.org/10.1088/1475-7516/2018/07/009</a>',
            )

            self.assertRegex(
                broad_doi_fn("10.1088/1475-7516/2019/02/E02/meta"),
                r'<a.*href="https://.*10.1088/1475-7516/2019/02/E02/meta".*>https://doi.org/10.1088/1475-7516/2019/02/E02/meta</a>',
            )
            self.assertRegex(
                broad_doi_fn("10.1088/1475-7516/2019/02/E02/META"),
                r'<a.*href="https://.*10.1088/1475-7516/2019/02/E02/META".*>https://doi.org/10.1088/1475-7516/2019/02/E02/META</a>',
            )
            self.assertRegex(
                broad_doi_fn("doi:10.1088/1475-7516/2018/07/009"),
                r'<a.*href="https://.*10.1088/1475-7516/2018/07/009".*>https://doi.org/10.1088/1475-7516/2018/07/009</a>',
            )

            self.assertRegex(
                broad_doi_fn("doi:10.1088/1475-7516/2019/02/E02/meta"),
                r'<a.*href="https://.*10.1088/1475-7516/2019/02/E02/meta".*>https://doi.org/10.1088/1475-7516/2019/02/E02/meta</a>',
            )
            self.assertRegex(
                broad_doi_fn("doi:10.1088/1475-7516/2019/02/E02/META"),
                r'<a.*href="https://.*10.1088/1475-7516/2019/02/E02/META".*>https://doi.org/10.1088/1475-7516/2019/02/E02/META</a>',
            )

            txt = broad_doi_fn(
                "10.1088/1475-7516/2018/07/009 10.1088/1475-7516/2019/02/E02/meta"
            )
            self.assertNotRegex(
                txt,
                r"this.*URL",
                'DOIs should not get the generic "this https URL" they should have the DOI text',
            )
            self.assertRegex(
                txt,
                r"<a.*>https://doi.org/10.1088/1475-7516/2018/07/009</a> <a.*>https://doi.org/10.1088/1475-7516/2019/02/E02/meta</a>",
                "Should handle two DOIs in a row correctly",
            )

            urlized_doi = broad_doi_fn(
                "10.1175/1520-0469(1996)053<0946:ASTFHH>2.0.CO;2"
            )

            self.assertNotIn(
                'href="http://2.0.CO"',
                urlized_doi,
                "Should not have odd second <A> tag for DOI urlize for ao-sci/9503001 see ARXIVNG-2049",
            )

            leg_rx = r'<a.* href="https://doi.org/10.1175/1520-0469%281996%29053%3C0946%3AASTFHH%3E2.0.CO%3B2".*'
            self.assertRegex(
                urlized_doi,
                leg_rx,
                "Should handle Complex DOI for ao-sci/9503001 see ARXIVNG-2049",
            )

            # in legacy:
            # <a href="/ct?url=https%3A%2F%2Fdx.doi.org%2F10.1175%252F1520-0469%25281996%2529053%253C0946%253AASTFHH%253E2.0.CO%253B2&amp;v=34a1af05">10.1175/1520-0469(1996)053&lt;0946:ASTFHH&gt;2.0.CO;2</a>

            # Post clickthrough on legacy goes to :
            # https://dx.doi.org/10.1175%2F1520-0469%281996%29053%3C0946%3AASTFHH%3E2.0.CO%3B2

    def test_dont_urlize_category_name(self):
        with self.app.app_context():
            urlize = links.urlizer()
            self.assertEqual(
                urlize("math.CO"),
                "math.CO",
                "category name math.CO should not get urlized",
            )
            self.assertIn(
                'href="http://supermath.co',
                urlize("supermath.co"),
                "hostname close to category name should get urlized",
            )

    def test_dont_urlize_arxiv_dot_org(self):
        with self.app.app_context():
            urlize = links.urlizer()

            self.assertNotRegex(
                urlize("something https://arxiv.org bla"),
                r"this.*URL",
                'arxiv.org should not get "this https URL" ARXIVNG-2130',
            )

            self.assertNotRegex(
                urlize("something arxiv.org bla"),
                r"this.*URL",
                'arxiv.org should not get "this https URL" ARXIVNG-2130',
            )

            self.assertRegex(
                urlize("something arXiv.org"),
                r"arXiv\.org",
                "arXiv.org should be preserved as text [ARXIVNG-2130]",
            )

            self.assertRegex(
                urlize("something arxiv.org"),
                r"arxiv\.org",
                "arXiv.org should be preserved as text [ARXIVNG-2130]",
            )



"""Tests version tag."""
from unittest import TestCase

from arxiv.release.tag_check import is_valid_python_public_version


class TestVersions(TestCase):

    def test_python_version(self):
        self.assertTrue(is_valid_python_public_version('1.2.0'))
        self.assertTrue(is_valid_python_public_version('1.2'))
        self.assertTrue(is_valid_python_public_version('1.2.3'))
        self.assertTrue(is_valid_python_public_version('1.2.30'))
        self.assertTrue(is_valid_python_public_version('1.2.1rc2'))
        self.assertTrue(is_valid_python_public_version('1.2.1rc23'))

        self.assertTrue(is_valid_python_public_version('1.2.1.post2'))
        self.assertTrue(is_valid_python_public_version('1.2.1.dev234'))
        self.assertTrue(is_valid_python_public_version('1.2.post1'))
        self.assertTrue(is_valid_python_public_version('1.2.dev1'))

        self.assertTrue(is_valid_python_public_version('1.2.1b1'))
        self.assertTrue(is_valid_python_public_version('1.2.1a1'))
        self.assertTrue(is_valid_python_public_version('1.2b1'))
        self.assertTrue(is_valid_python_public_version('1.2a1'))

        self.assertTrue(is_valid_python_public_version('1.2.0-rc1'))
        self.assertFalse(is_valid_python_public_version('1.2.0ll2k-@#$'))
        self.assertFalse(is_valid_python_public_version('xx.2.0ll2k-@#$'))
        self.assertFalse(is_valid_python_public_version('1.2.0-rc1-pylocalstuff'))



"""Fuzz test for :mod:`arxiv.base.urls.clickthrough`."""

import unittest

from hypothesis import given
from hypothesis.strategies import text

from ..clickthrough import create_hash, is_hash_valid


class TestClickthroughFuzzTest(unittest.TestCase):
    """Fuzz tests for :func:`.create_hash` and :func:`.is_hash_valid`."""

    @given(text(), text())
    def test_clickthrough(self, s, v):
        """Generate a hash for a bunch of "urls" and a bunch of hashes."""
        h = create_hash(s, v)
        self.assertTrue(is_hash_valid(s, v, h))



"""Tests for author and affiliation parsing."""
from unittest import TestCase

from ..authors import parse_author_affil, split_authors


class TestAuthorAffiliationParsing(TestCase):

    def test_split_authors(self):
        self.assertListEqual(split_authors('Simeon Warner'), ['Simeon Warner'])

        self.assertListEqual(
            split_authors('The DELPHI Collaboration, J. Abdallah, et al'),
            ['The DELPHI Collaboration', ',', 'J. Abdallah', ',', 'et al']
        )

        self.assertListEqual(
            split_authors('BELLE Collaboration: A Person, Nother Person'),
            ['BELLE Collaboration', ':', 'A Person', ',', 'Nother Person']
        )

        self.assertListEqual(
            split_authors('Simeon Warner, Herbert Van de Sompel'),
            ['Simeon Warner', ',', 'Herbert Van de Sompel']
        )

        self.assertListEqual(
            split_authors('An Author'),
            ['An Author'],
            'single author'
        )

        self.assertListEqual(
            split_authors(''),
            [],
            'empty author'
        )

        self.assertListEqual(
            split_authors('An Author (affil)'),
            ['An Author', '(affil)'],
            'single author with affil'
        )
        self.assertListEqual(
            split_authors('An Author     (affil)'),
            ['An Author', '(affil)'],
            'single author with affil'
        )
        self.assertListEqual(
            split_authors('An Author and Another P. H. J. Author (affil)'),
            ['An Author', ',', 'Another P. H. J. Author', '(affil)'],
            'double author with affil'
        )
        self.assertListEqual(
            split_authors(
                'John Von Neumann, Herbert Van de Sompel, Fred Bloggs, Jr, et al'),
            ['John Von Neumann', ',', 'Herbert Van de Sompel',
             ',', 'Fred Bloggs, Jr', ',', 'et al'],
            'multiple with prefixes and suffixes'
        )
        self.assertListEqual(
            split_authors('sum won ( whatever affil  data   unmunged  )'),
            ['sum won', '( whatever affil data unmunged )'],
            'one author, two labs'
        )
        self.assertListEqual(
            split_authors('sum won(1,2)((1)lab a,(2)lab b)'),
            ['sum won', '(1,2)', '((1)lab a,(2)lab b)'],
            'one author, two labs'
        )

    def test_parse_author_affil_basic(self):
        self.assertListEqual(parse_author_affil(
            'Simeon Warner'), [['Warner', 'Simeon', '']])

        self.assertListEqual(
            parse_author_affil('Simeon Warner Jr'),
            [['Warner', 'Simeon', 'Jr']])

        self.assertListEqual(
            parse_author_affil('Simeon Warner Jr.'),
            [['Warner', 'Simeon', 'Jr.']])

        self.assertListEqual(
            parse_author_affil('Simeon Warner Sr'),
            [['Warner', 'Simeon', 'Sr']])

        self.assertListEqual(
            parse_author_affil('Simeon Warner Sr.'),
            [['Warner', 'Simeon', 'Sr.']])

        self.assertListEqual(
            parse_author_affil('SM Warner'),
            [['Warner', 'SM', '']])

        self.assertListEqual(
            parse_author_affil('SM. Warner'),
            [['Warner', 'SM.', '']])

    def test_parse_author_affil_basic2(self):
        self.assertListEqual(
            parse_author_affil('S.M. Warner'),
            [['Warner', 'S. M.', '']])

        self.assertListEqual(
            parse_author_affil('John Von Neumann'),
            [['Von Neumann', 'John', '']])

        self.assertListEqual(
            parse_author_affil('Herbert Van de Sompel'),
            [['Van de Sompel', 'Herbert', '']])

        self.assertListEqual(
            parse_author_affil('del Norte'),
            [['Norte', 'del', '']])

        self.assertListEqual(
            parse_author_affil('Fred del Norte'),
            [['del Norte', 'Fred', '']])

        self.assertListEqual(
            parse_author_affil('BELLE'),
            [['BELLE', '', '']])

        self.assertListEqual(
            parse_author_affil('BELLE Collaboration: A Person, Nother Person'),
            [
                ['BELLE Collaboration', '', ''],
                ['Person', 'A', ''],
                ['Person', 'Nother', '']
            ])

        self.assertListEqual(
            parse_author_affil('The DELPHI Collaboration, J. Abdallah, et al'),
            [['The DELPHI Collaboration', '', ''], ['Abdallah', 'J.', '']]
        )

        self.assertListEqual(
            parse_author_affil('Ali Vaziri Astaneh, Federico Fuentes'),
            [['Vaziri Astaneh', 'Ali', ''], ['Fuentes', 'Federico', '']]
        )

    def test_parse_author_affil_with_affiliations(self):
        self.assertListEqual(
            parse_author_affil('sum won (lab a)'),
            [['won', 'sum', '', 'lab a']])

        self.assertListEqual(
            parse_author_affil('sum won (lab a; lab b)'),
            [['won', 'sum', '', 'lab a; lab b']])

        self.assertListEqual(
            parse_author_affil('sum won (lab a, lab b)'),
            [['won', 'sum', '', 'lab a, lab b']])

        self.assertListEqual(
            parse_author_affil('sum won (1,2) ( (1) lab a, (2) lab b)'),
            [['won', 'sum', '', 'lab a', 'lab b']])

        self.assertListEqual(
            parse_author_affil('sum won(1,2)((1)lab a,(2)lab b)'),
            [['won', 'sum', '', 'lab a', 'lab b']])

        self.assertListEqual(
            parse_author_affil('a.b.first, c.d.second (affil)'),
            [['first', 'a. b.', '', 'affil'],
             ['second', 'c. d.', '', 'affil']])

        self.assertListEqual(
            parse_author_affil('a.b.first (affil), c.d.second (affil)'),
            [['first', 'a. b.', '', 'affil'],
             ['second', 'c. d.', '', 'affil']])

        self.assertListEqual(
            parse_author_affil(
                'a.b.first, c.d.second (1), e.f.third, g.h.forth (2,3) ((1) affil1, (2) affil2, (3) affil3)'
            ),
            [
                ['first', 'a. b.', '', 'affil1'],
                ['second', 'c. d.', '', 'affil1'],
                ['third', 'e. f.', '', 'affil2', 'affil3'],
                ['forth', 'g. h.', '', 'affil2', 'affil3']
            ])

        self.assertListEqual(
            parse_author_affil((
                "QUaD collaboration: S. Gupta (1), P. Ade (1), J. Bock (2,3), M. Bowden "
                "(1,4), M. L. Brown (5), G. Cahill (6), P. G. Castro (7,8), S. Church (4), T. "
                "Culverhouse (9), R. B. Friedman (9), K. Ganga (10), W. K. Gear (1), J. "
                "Hinderks (5,11), J. Kovac (3), A. E. Lange (4), E. Leitch (2,3), S. J. "
                "Melhuish (12), Y. Memari (7), J. A. Murphy (6), A. Orlando (1,3), C. "
                "O'Sullivan (6), L. Piccirillo (12), C. Pryke (9), N. Rajguru (1,13), B. "
                "Rusholme (4,14), R. Schwarz (9), A. N. Taylor (7), K. L. Thompson (4), A. H. "
                "Turner (1), E. Y. S. Wu (4), M. Zemcov (1,2,3) ((1) Cardiff University, (2) "
                "JPL, (3) Caltech, (4) Stanford University, (5) University of Cambridge, (6) "
                "National University of Ireland Maynooth, (7) University of Edinburgh, (8) "
                "Universidade Tecnica de Lisboa, (9) University of Chicago, (10) Laboratoire "
                "APC/CNRS, (11) NASA Goddard, (12) University of Manchester, (13) UCL, (14) "
                "IPAC) "
            )),
            [
                ["QUaD collaboration", "", ""],
                ["Gupta", "S.", "", "Cardiff University"],
                ["Ade", "P.", "", "Cardiff University"],
                ["Bock", "J.", "", "JPL", "Caltech"],
                ["Bowden", "M.", "", "Cardiff University", "Stanford University"],
                ["Brown", "M. L.", "", "University of Cambridge"],
                ["Cahill", "G.", "", "National University of Ireland Maynooth"],
                [
                    "Castro", "P. G.", "",
                    "University of Edinburgh",
                    "Universidade Tecnica de Lisboa"
                ],
                ["Church", "S.", "", "Stanford University"],
                ["Culverhouse", "T.", "", "University of Chicago"],
                ["Friedman", "R. B.", "", "University of Chicago"],
                ["Ganga", "K.", "", "Laboratoire APC/CNRS"],
                ["Gear", "W. K.", "", "Cardiff University"],
                ["Hinderks", "J.", "", "University of Cambridge", "NASA Goddard"],
                ["Kovac", "J.", "", "Caltech"],
                ["Lange", "A. E.", "", "Stanford University"],
                ["Leitch", "E.", "", "JPL", "Caltech"],
                ["Melhuish", "S. J.", "", "University of Manchester"],
                ["Memari", "Y.", "", "University of Edinburgh"],
                ["Murphy", "J. A.", "", "National University of Ireland Maynooth"],
                ["Orlando", "A.", "", "Cardiff University", "Caltech"],
                ["O'Sullivan", "C.", "", "National University of Ireland Maynooth"],
                ["Piccirillo", "L.", "", "University of Manchester"],
                ["Pryke", "C.", "", "University of Chicago"],
                ["Rajguru", "N.", "", "Cardiff University", "UCL"],
                ["Rusholme", "B.", "", "Stanford University", "IPAC"],
                ["Schwarz", "R.", "", "University of Chicago"],
                ["Taylor", "A. N.", "", "University of Edinburgh"],
                ["Thompson", "K. L.", "", "Stanford University"],
                ["Turner", "A. H.", "", "Cardiff University"],
                ["Wu", "E. Y. S.", "", "Stanford University"],
                ["Zemcov", "M.", "", "Cardiff University", "JPL", "Caltech"]
            ],
            'parse_author_affil (mind-blowing) 1/1'
        )

        # Problem case with 1110.4366
        self.assertListEqual(
            parse_author_affil(
                'Matthew Everitt, Robert M. Heath and Viv Kendon'),
            [['Everitt', 'Matthew', ''],
             ['Heath', 'Robert M.', ''],
             ['Kendon', 'Viv', '']],
            'parse_author_affil for 1110.4366'
        )

        # look like bugs, but aren't
        self.assertListEqual(
            parse_author_affil('sum won ((lab a), (lab b))'),
            [['won', 'sum', '']],
            'parse_author_affil (bug imposter) 1/2'
        )
        self.assertListEqual(
            parse_author_affil('sum won ((lab a) (lab b))'),
            [['won', 'sum', '']],
            'parse_author_affil (bug imposter) 2/2'
        )

        self.assertListEqual(
            parse_author_affil('Anatoly Zlotnik and Jr-Shin Li'),
            [['Zlotnik', 'Anatoly', ''],
             ['Li', 'Jr-Shin', '']],
            'jr issue (Anatoly Zlotnik and Jr-Shin Li)'
        )

        self.assertListEqual(
            # 2106.08865; back propagation with comma (1,2), (4,5)
            parse_author_affil(
                "E. Trigueros P\'aez (1 and 2) and R. H. Barb\'a (3) and I. Negueruela (1) "
                "and J. Ma\'iz Apell\'aniz (2) and S. Sim\'on-D\'iaz (4 and 5) and G.Holgado (2) "
                "((1) Departamento de F\'isica Aplicada - Universidad de Alicante, (2) "
                "Centro de Astrobiolog\'ia - CSIC-INTA, (3) Departamento de Astronom\'ia - Universidad "
                "de La Serena, (4) Instituto de Astrof\'isica de Canarias, (5) Departamento "
                "de Astrof\'isica - Universidad de La Laguna)"),            [["P'aez", 'E. Trigueros', '', "Departamento de F'isica Aplicada - Universidad de Alicante", "Centro de Astrobiolog'ia - CSIC-INTA"],
             ["Barb'a", 'R. H.', '', "Departamento de Astronom'ia - Universidad de La Serena"],
             ['Negueruela', 'I.', '', "Departamento de F'isica Aplicada - Universidad de Alicante"],
             ["Apell'aniz", "J. Ma'iz", '', "Centro de Astrobiolog'ia - CSIC-INTA"],
             ["Sim'on-D'iaz", 'S.', '', "Instituto de Astrof'isica de Canarias", "Departamento de Astrof'isica - Universidad de La Laguna"],
             ['Holgado', 'G.', '', "Centro de Astrobiolog'ia - CSIC-INTA"]],
             'back propagation for 2106.08865 with comma as separator'
        )

        self.assertListEqual(
            # 2106.08865; back propagation with "and", e.g. Some Author (1 and 2), Another Author (4 and 5)
            parse_author_affil(
                "E. Trigueros P\'aez (1 and 2) and R. H. Barb\'a (3) and I. Negueruela (1) "
                "and J. Ma\'iz Apell\'aniz (2) and S. Sim\'on-D\'iaz (4 and 5) and G.Holgado (2) "
                "((1) Departamento de F\'isica Aplicada - Universidad de Alicante, (2) "
                "Centro de Astrobiolog\'ia - CSIC-INTA, (3) Departamento de Astronom\'ia - Universidad "
                "de La Serena, (4) Instituto de Astrof\'isica de Canarias, (5) Departamento "
                "de Astrof\'isica - Universidad de La Laguna)"),
            [["P'aez", 'E. Trigueros', '', "Departamento de F'isica Aplicada - Universidad de Alicante", "Centro de Astrobiolog'ia - CSIC-INTA"],
             ["Barb'a", 'R. H.', '', "Departamento de Astronom'ia - Universidad de La Serena"],
             ['Negueruela', 'I.', '', "Departamento de F'isica Aplicada - Universidad de Alicante"],
             ["Apell'aniz", "J. Ma'iz", '', "Centro de Astrobiolog'ia - CSIC-INTA"],
             ["Sim'on-D'iaz", 'S.', '', "Instituto de Astrof'isica de Canarias", "Departamento de Astrof'isica - Universidad de La Laguna"],
             ['Holgado', 'G.', '', "Centro de Astrobiolog'ia - CSIC-INTA"]],
             'back propagation for 2106.08865 with "and" as separator'
        )


        # ====== Extra tests for arXiv::AuthorAffil ARXIVDEV-728 ======

        # [parse_author_affil]
        self.assertListEqual(
            parse_author_affil(''),
            [],
            'parse_author_affil (empty)'
        )
        self.assertListEqual(
            parse_author_affil('Simeon Warner Jr'),
            [['Warner', 'Simeon', 'Jr']],
            'parse_author_affil (basic) 2/12'
        )
        self.assertListEqual(
            parse_author_affil('BELLE Collaboration'),
            [['BELLE Collaboration', '', '']],
            'parse_author_affil (lone "BELLE Collaboration") 2/3'
        )

        self.assertListEqual(
            parse_author_affil('BELLE Collaboration'),
            [['BELLE Collaboration', '', '']],
            'parse_author_affil (lone "BELLE Collaboration") 2/3'
        )



"""Tests for :mod:`arxiv.base.urls`."""

from unittest import TestCase, mock
from flask import Flask, url_for, Blueprint
from arxiv.base import urls, Base
from arxiv.base.exceptions import ConfigurationError
from werkzeug.routing import BuildError


class TestStaticURLs(TestCase):
    """Test building static URLs."""

    def test_static_urls(self):
        """We have vanilla Flask app with a blueprint."""
        self.app = Flask('test')
        self.app.config['SERVER_NAME'] = 'nope.com'
        Base(self.app)

        self.app.register_blueprint(Blueprint('fooprint', __name__,
                                              url_prefix='/foo',
                                              static_folder='static',
                                              static_url_path='baz'))

        with self.app.app_context():
            url = url_for('base.static', filename='css/abs.css')
            self.assertTrue(url.startswith('http://nope.com/static/base/'))
            url = url_for('fooprint.static', filename='img/foo.jpg')
            self.assertTrue(url.startswith('http://nope.com/foo/static/test/'))

    def test_relative_static_urls(self):
        """Relative static paths are enabled."""
        self.app = Flask('test')
        self.app.config['SERVER_NAME'] = 'nope.com'
        self.app.config.update({
            'SITE_HUMAN_NAME': 'The test site of testiness',
            'SITE_HUMAN_SHORT_NAME': 'Test site',
            'RELATIVE_STATIC_PATHS': True,
            'RELATIVE_STATIC_PREFIX': 'oof',
            'SITE_URL_PREFIX': '/test'
        })
        Base(self.app)

        self.app.register_blueprint(Blueprint('fooprint', __name__,
                                              url_prefix='/foo',
                                              static_folder='static',
                                              static_url_path='baz'))

        with self.app.app_context():
            url = url_for('base.static', filename='css/abs.css')
            self.assertTrue(url.startswith('http://nope.com/oof/static/base/'),
                            'The static URL for base starts with the'
                            ' configured prefix.')
            url = url_for('fooprint.static', filename='img/foo.jpg')
            self.assertFalse(url.startswith('http://nope.com/oof'),
                             'The blueprint static path is not affected.')


class TestGetURLMap(TestCase):
    """Tests for :func:`arxiv.base.urls.get_url_map`."""

    @mock.patch('arxiv.base.urls.base_config', mock.MagicMock(URLS=[]))
    def test_no_urls_configured(self):
        """No external URLs are defined on the current app nor in base."""
        app = mock.MagicMock(config={'URLS': []})
        adapter = urls.build_adapter(app)
        self.assertEqual(len(adapter.map._rules), 0,
                         "No URL patterns are loaded")

    @mock.patch('arxiv.base.urls.base_config',
                mock.MagicMock(URLS=[
                    ('foo', '/foo', 'foo.org'),
                    ('bar', '/bar/<string:foo>', 'bar.org'),
                ]))
    def test_base_urls(self):
        """Only URLs are defined in base; the current app has none."""
        app = mock.MagicMock(config={'URLS': []})
        adapter = urls.build_adapter(app)
        self.assertEqual(len(adapter.map._rules), 2,
                         "Only base URLs are loaded")

    @mock.patch('arxiv.base.urls.base_config',
                mock.MagicMock(URLS=[
                    ('foo', '/foo', 'foo.org'),
                    ('bar', '/bar/<string:foo>', 'bar.org'),
                ]))
    def test_combined_urls(self):
        """Only non-overlapping URLs are defined in base and the app."""
        app = mock.MagicMock(config={
            'URLS': [
                ('baz', '/baz', 'baz.org'),
                ('bat', '/bat/<string:foo>', 'bat.org'),
            ]
        })
        adapter = urls.build_adapter(app)
        self.assertEqual(len(adapter.map._rules), 4, "All URLs are loaded")

    @mock.patch('arxiv.base.urls.base_config', mock.MagicMock(URLS=[
                    ('baz', '/foo', 'foo.org'),
                    ('bar', '/bar/<string:foo>', 'bar.org'),
                ]))
    def test_overlapping_urls(self):
        """Overlapping URLs are defined in base and the app."""
        app = mock.MagicMock(config={
            'URLS': [
                ('baz', '/baz', 'baz.org'),
                ('bat', '/bat/<string:foo>', 'bat.org'),
            ]
        })
        adapter = urls.build_adapter(app)
        self.assertEqual(len(adapter.map._rules), 3,
                         "Duplicate URLs not added")
        self.assertEqual(adapter.build('baz'), 'https://baz.org/baz',
                         "The application config is preferred to base")


class TestExternalURLFallback(TestCase):
    """Test external URL building in a Flask app."""

    def setUp(self):
        """Create a Flask app."""
        self.app = Flask('test')
        self.app.config['URLS'] = [
            ('baz', '/baz', 'baz.org'),
            ('bat', '/bat/<string:foo>', 'bat.org'),
        ]
        self.app.config['SERVER_NAME'] = 'nope.com'

        @self.app.route('/something')
        def something():
            return 'nothing'

    def test_application_url(self):
        """url_for works as expected for an app-defined URL."""
        Base(self.app)
        with self.app.app_context():
            self.assertEqual(url_for('something'), 'http://nope.com/something')

    @mock.patch('arxiv.base.urls.base_config',
                mock.MagicMock(URLS=[
                    ('baz', '/foo', 'foo.org'),
                    ('bar', '/bar/<string:foo>', 'bar.org'),
                ]))
    def test_url_from_app_config(self):
        """url_for falls back to URLs from the application config."""
        Base(self.app)
        with self.app.app_context():
            self.assertEqual(url_for('bat', foo='yes'),
                             'https://bat.org/bat/yes')
            self.assertEqual(url_for('baz'), 'https://baz.org/baz')

    @mock.patch('arxiv.base.urls.base_config', mock.MagicMock(URLS=[
                    ('baz', '/foo', 'foo.org'),
                    ('bar', '/bar/<string:foo>', 'bar.org'),
                ]))
    def test_url_from_base_config(self):
        """url_for falls back to URLs from the application config."""
        Base(self.app)
        with self.app.app_context():
            self.assertEqual(url_for('bar', foo=1), 'https://bar.org/bar/1')


class TestWithClient(TestCase):
    """Test external URL building in a Flask app."""

    def setUp(self):
        """Create a Flask app."""
        self.app = Flask('test')
        self.app.config['URLS'] = [
            ('baz', '/baz', 'baz.org'),
            ('bat', '/bat/<string:foo>', 'bat.org'),
        ]
        self.app.config['SERVER_NAME'] = 'nope.com'
        Base(self.app)

        @self.app.route('/baz_location')
        def baz_location():
            return url_for('baz')

        @self.app.route('/bat_location')
        def bat_location():
            return url_for('bat', foo=1)

        @self.app.route('/acknowledgment_location')
        def acknowledgment_location():
            return url_for('acknowledgment')

        @self.app.route('/something')
        def something():
            return 'nothing'

        self.client = self.app.test_client()

    def test_application_url(self):
        """url_for works as expected for an app-defined URL."""
        self.assertEqual(self.client.get('/baz_location').data,
                         b'https://baz.org/baz')
        self.assertEqual(self.client.get('/bat_location').data,
                         b'https://bat.org/bat/1')
        self.assertEqual(self.client.get('/acknowledgment_location').data,
                         b'https://arxiv.org/about/ourmembers')



from unittest import TestCase
from io import StringIO

import logging as pyLogging
from arxiv.base import logging


class TestGetLogger(TestCase):
    """Test :func:`.logging.getLogger`."""

    def test_get_logger_no_app_nor_request(self):
        """There is no application nor request context."""
        stream = StringIO()

        logger = logging.getLogger('foologger', stream)
        self.assertIsInstance(logger, pyLogging.Logger,
                              "Should return a logging.Logger instance")

        logger.error('foo')
        captured_value = stream.getvalue()
        stream.close()
        self.assertIn('ERROR: "foo"', captured_value,
                      "Should log normally even if request is not present")



"""Test sending email."""

from unittest import TestCase, mock
from .. import mail


class TestSendEmail(TestCase):
    """Tests for :func:`mail.send`."""

    @mock.patch(f'{mail.__name__}.smtplib.SMTP')
    def test_send_text(self, mock_SMTP):
        """Send a text-only e-mail."""
        mock_SMTP_instance = mock.MagicMock()
        mock_SMTP.return_value.__enter__.return_value = mock_SMTP_instance
        mail.send("erick.peirson@gmail.com", "Subject!", "Hi there.")
        expected = [
            'Subject: Subject!',
            'From: noreply@arxiv.org',
            'To: erick.peirson@gmail.com',
            'Content-Type: text/plain; charset="utf-8"',
            'Content-Transfer-Encoding: 7bit',
            'MIME-Version: 1.0',
            'Hi there.',
        ]
        content = str(mock_SMTP_instance.send_message.call_args[0][0])
        for part in expected:
            self.assertIn(part, content)

    @mock.patch(f'{mail.__name__}.smtplib.SMTP')
    def test_send_text_with_cc_and_bcc(self, mock_SMTP):
        """Send a text-only e-mail with CC and BCC recipients."""
        mock_SMTP_instance = mock.MagicMock()
        mock_SMTP.return_value.__enter__.return_value = mock_SMTP_instance
        mail.send("erick.peirson@gmail.com", "Subject!", "Hi there.",
                  cc_recipients=['foo@somewhere.edu', 'baz@somewhere.edu'],
                  bcc_recipients=['bar@elsewhere.edu'])
        expected = [
            'Subject: Subject!',
            'From: noreply@arxiv.org',
            'To: erick.peirson@gmail.com',
            'CC: foo@somewhere.edu, baz@somewhere.edu',
            'BCC: bar@elsewhere.edu',
            'Content-Type: text/plain; charset="utf-8"',
            'Content-Transfer-Encoding: 7bit',
            'MIME-Version: 1.0',
            'Hi there.',
        ]
        content = str(mock_SMTP_instance.send_message.call_args[0][0])
        for part in expected:
            self.assertIn(part, content)

    @mock.patch(f'{mail.__name__}.smtplib.SMTP')
    def test_send_text_and_html(self, mock_SMTP):
        """Send an email with text and HTML."""
        mock_SMTP_instance = mock.MagicMock()
        mock_SMTP.return_value.__enter__.return_value = mock_SMTP_instance
        html_body = "<html><body><h1>Hi</h1><p>there!</p></body></html>"
        mail.send("erick.peirson@gmail.com", "Subject!", "Hi there.",
                  html_body=html_body)
        sent_message = str(mock_SMTP_instance.send_message.call_args[0][0])
        expected_headers = [
            'Subject: Subject!',
            'From: noreply@arxiv.org',
            'To: erick.peirson@gmail.com',
            'MIME-Version: 1.0',
            'Content-Type: multipart/alternative;'
        ]
        text_part = [
            'Content-Type: text/plain; charset="utf-8"',
            'Content-Transfer-Encoding: 7bit',
            'Hi there.'
        ]
        html_part = [
            'Content-Type: text/html; charset="utf-8"',
            'Content-Transfer-Encoding: 7bit',
            'MIME-Version: 1.0',
            '<html><body><h1>Hi</h1><p>there!</p></body></html>'
        ]
        for header in expected_headers:
            self.assertIn(header, sent_message)
        for part in text_part:
            self.assertIn(part, sent_message)
        for part in html_part:
            self.assertIn(part, sent_message)



"""Tests for Tex to UTF8 parsing."""
from unittest import TestCase

from ..tex2utf import tex2utf


class TextTex2Utf(TestCase):

    def test_tex2utf(self):
        test_str = "abc def ghijk lmnop qrs tuv wxyz 1234567890 !@# $%^ &* () _-=+"
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, test_str)

        utf_out = tex2utf(test_str, greek=False)
        self.assertEqual(utf_out, test_str)

        test_str = "\\'e"
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, chr(0xe9))

        test_str = "\\'E"
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, chr(
            0xc9))

        test_str = "\\'E"
        utf_out = tex2utf(test_str, greek=True)
        self.assertEqual(utf_out, chr(
            0xc9))

        test_str = "\\'E"
        utf_out = tex2utf(test_str, greek=False)
        self.assertEqual(utf_out, chr(0xc9))

        test_str = "\\'E"
        utf_out = tex2utf(test_str, False)
        self.assertEqual(utf_out, chr(0xc9))

        # single textsymbol
        test_str = '\\OE'
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, chr(0x0152))

        # single textsymbol followed by newline
        test_str = "\\OE\n"
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, chr(0x0152) + "\n")

        # test_string of textsymbols
        test_str = "\\OE\\S"
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, chr(0x0152) + chr(0x00a7))

        # test_string of textsymbols followed by newline
        test_str = "\\OE\\S\n"
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, chr(0x0152) + chr(
            0x00a7) + "\n")

        # combination of textlet and textsymbols with whitespace as separator
        test_str = "\\ddag \\OE\\S"
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, chr(0x2021) + chr(0x0152) + chr(
            0x00a7))

        # single greek textlet
        test_str = '\\alpha'
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, chr(
            0x03b1))

        test_str = '\\alpha'
        utf_out = tex2utf(test_str, True)
        self.assertEqual(utf_out, chr(
            0x03b1))

        test_str = '\\alpha'
        utf_out = tex2utf(test_str, False)
        self.assertEqual(utf_out, r'\alpha')

        # simple test_string of greek
        test_str = '\\alpha\\beta\gamma'
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, chr(0x03b1) + chr(0x03b2) + chr(
            0x03b3))

        # test_string of greek textlet with nested curlies
        test_str = '\\alpha{\\beta{\gamma}}'
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, chr(
            0x03b1) + '{' + chr(0x03b2) + chr(0x03b3) + '}')

        # another test_string of greek with nested curlies
        test_str = '\\alpha{\\beta{\gamma}\macro}'
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, chr(
            0x03b1) + '{' + chr(0x03b2) + chr(0x03b3) + '\macro}')

        # use "\ " as textlet delimiter
        test_str = 'foo \\alpha\ bar'
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, 'foo ' + chr(0x03b1) +
                         ' bar')

        # use "\ " as textlet delimiter
        test_str = '\\alpha\ \\beta{something}'
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, chr(0x03b1) + ' ' +
                         chr(0x03b2) + '{something}')

        # use "\ " as textlet delimiter
        test_str = 'foo \OE\ bar'
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, 'foo ' + chr(0x0152) +
                         ' bar')

        # use " " as textlet delimiter
        test_str = 'foo \\alpha bar'
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, 'foo ' + chr(0x03b1) + 'bar')

        # use empty "{}" as textlet delimiter
        test_str = 'foo \\alpha{}bar'
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, 'foo ' + chr(0x03b1) + 'bar')

        # textlet followed by non-empty "{ +  + }"
        test_str = 'foo \\alpha{-}bar'
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, 'foo ' + chr(0x03b1) +
                         '{-}bar')

        # textlet followed by underscore (for subscript)
        test_str = 'foo \\alpha_7'
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, 'foo ' + chr(0x03b1) +
                         '_7')

    def test_tex2utf_underscore(self):
        # textlet followed by underscore (for subscript)
        test_str = 'foo \\alpha_\\beta_\\gamma_7'
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, 'foo ' + chr(0x03b1) + '_' +
                         chr(0x03b2) + '_' + chr(0x03b3) + '_7')

    # <test><args>\'E</args><func>tex2UTF</func><out>&#xc9;</out></test>
    def test_tex2utf_curly(self):
        test_str = "\\'{e}"
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, chr(
            0xe9))

        # <test><args>\'{e}</args><func>tex2UTF</func><out>&#xe9;</out></test>
        # <test><args>{\'e}</args><func>tex2UTF</func><out>{&#xe9;}</out></test>

    def test_ARXIVDEV2322fixes(self):
        test_str = "ARXIVDEV-2322 \\u{A} fix"
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, 'ARXIVDEV-2322 ' + chr(0x102) + ' fix')

        test_str = "ARXIVDEV-2322 \\u{a} fix"
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, 'ARXIVDEV-2322 ' + chr(0x103) + ' fix')

        test_str = "ARXIVDEV-2322 \\u{O} fix"
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, 'ARXIVDEV-2322 ' + chr(0x14e) + ' fix')

        test_str = "ARXIVDEV-2322 \\u{o} fix"
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, 'ARXIVDEV-2322 ' + chr(0x14f) + ' fix')

        test_str = "ARXIVDEV-2322 \\k{i} fix"
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, 'ARXIVDEV-2322 ' + chr(0x12f) + ' fix')

        test_str = "ARXIVDEV-2322 \\v{g} fix"
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, 'ARXIVDEV-2322 ' + chr(0x1e7) + ' fix')

        test_str = "ARXIVDEV-2322 \\c{g} fix"
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, 'ARXIVDEV-2322 ' + chr(0x123) + ' fix')

        test_str = "ARXIVDEV-2322 \\DJ  fix"
        utf_out = tex2utf(test_str)
        self.assertEqual(utf_out, 'ARXIVDEV-2322 ' + chr(0x110) + ' fix')

    def test_ARXIVOPS805(self):
        """{\aa} wasn't being converted correctly in /abs abstract field to å"""
        self.assertEqual(tex2utf("ARXIVOPS-805 \\aa  fix"),
                         'ARXIVOPS-805 å fix')

        self.assertEqual(tex2utf("ARXIVOPS-805 \\aa  fix", greek=False),
                         'ARXIVOPS-805 å fix')

        self.assertEqual(tex2utf("ARXIVOPS-805 \\aa  fix", greek=True),
                         'ARXIVOPS-805 å fix')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['arxiv',
 'arxiv.base',
 'arxiv.base.middleware',
 'arxiv.base.tests',
 'arxiv.base.urls',
 'arxiv.base.urls.tests',
 'arxiv.forms',
 'arxiv.identifier',
 'arxiv.integration',
 'arxiv.integration.api',
 'arxiv.integration.kinesis',
 'arxiv.integration.kinesis.consumer',
 'arxiv.license',
 'arxiv.mail',
 'arxiv.mail.tests',
 'arxiv.release',
 'arxiv.release.test',
 'arxiv.status',
 'arxiv.taxonomy',
 'arxiv.util',
 'arxiv.util.tests']

package_data = \
{'': ['*'],
 'arxiv.base': ['static/css/*',
                'static/fontawesome-svgs/*',
                'static/fontawesome-svgs/svgs/brands/*',
                'static/fontawesome-svgs/svgs/solid/*',
                'static/images/*',
                'static/images/icons/*',
                'static/js/*',
                'static/sass/*',
                'static/sass/bulma/*',
                'static/sass/bulma/css/*',
                'static/sass/bulma/sass/base/*',
                'static/sass/bulma/sass/components/*',
                'static/sass/bulma/sass/elements/*',
                'static/sass/bulma/sass/grid/*',
                'static/sass/bulma/sass/layout/*',
                'static/sass/bulma/sass/utilities/*',
                'static/sass/extensions/bulma-switch/*',
                'static_test/*',
                'templates/base/*'],
 'arxiv.mail': ['templates/*', 'templates/mail/*']}

install_requires = \
['aiohttp',
 'bleach',
 'boto3>=1.0.0,<2.0.0',
 'flask-s3',
 'flask>=2.2,<3.0',
 'jsonschema',
 'markupsafe',
 'pytz',
 'retry',
 'semantic-version',
 'typing-extensions',
 'uwsgi',
 'wtforms']

setup_kwargs = {
    'name': 'arxiv-base',
    'version': '1.0.0a1',
    'description': 'Common code for arXiv NG',
    'long_description': None,
    'author': 'arxiv.org',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<3.11',
}


setup(**setup_kwargs)



"""Tests for static URL modifications in :class:`.Base`."""

from unittest import TestCase, mock

from flask import Flask, url_for

from .. import Base


class TestAppWithStaticFiles(TestCase):
    """We are using :class:`.Base` on a Flask app."""

    def setUp(self):
        """Set up an app with static files."""
        self.static_url_path = '/a/static/path'
        self.host = 'foo.host'
        self.version = '0.3.2'
        self.app = Flask(__name__, static_url_path=self.static_url_path)
        self.app.config['APP_VERSION'] = self.version
        self.app.config['SERVER_NAME'] = self.host

    def test_base_alters_static_url(self):
        """When :class:`.Base` is applied, the static url path is changed."""
        with self.app.app_context():
            target_url = url_for('static', filename='foo.txt')

        self.assertEqual(
            target_url,
            f'http://{self.host}/a/static/path/foo.txt'
        )

        Base(self.app)

        with self.app.app_context():
            target_url = url_for('static', filename='foo.txt')

        self.assertEqual(
            target_url,
            f'http://{self.host}/static/{self.app.name}/{self.version}/foo.txt'
        )



"""Tests for :mod:`.serialize`."""

from unittest import TestCase, mock
from datetime import datetime, date
from pytz import UTC, timezone
from .. import serialize

ET = timezone('US/Eastern')


class TestISO8601JSONEncoder(TestCase):
    """
    Tests :func:`.serialize.dumps` and :class:`.serialize.ISO8601JSONEncoder`.
    """

    def test_encode_with_date(self):
        """Encode JSON with a :class:`.date` object."""
        self.assertEqual(
            serialize.dumps(date(year=1993, month=11, day=3)),
            '"1993-11-03"'
        )
        self.assertEqual(
            serialize.dumps({"date": date(year=1993, month=11, day=3)}),
            '{"date": "1993-11-03"}'
        )
        self.assertEqual(
            serialize.dumps([{"date": date(year=1993, month=11, day=3)}]),
            '[{"date": "1993-11-03"}]'
        )
        self.assertEqual(
            serialize.dumps([{"date": [date(year=1993, month=11, day=3)]}]),
            '[{"date": ["1993-11-03"]}]'
        )

    def test_encode_with_datetime(self):
        """Encode JSON with a :class:`.datetime` object."""
        self.assertEqual(
            serialize.dumps(datetime(year=1993, month=11, day=3)),
            '"1993-11-03T00:00:00"'
        )
        self.assertEqual(
            serialize.dumps({"date": datetime(year=1993, month=11, day=3)}),
            '{"date": "1993-11-03T00:00:00"}'
        )
        self.assertEqual(
            serialize.dumps([{"date": datetime(year=1993, month=11, day=3)}]),
            '[{"date": "1993-11-03T00:00:00"}]'
        )
        self.assertEqual(
            serialize.dumps([{"date": [datetime(year=1993, month=11, day=3)]}]),
            '[{"date": ["1993-11-03T00:00:00"]}]'
        )

    def test_encode_with_datetime_tz(self):
        """Encode JSON with a TZ-aware :class:`.datetime` object."""
        self.assertEqual(
            serialize.dumps(datetime(year=1993, month=11, day=3, tzinfo=ET)),
            '"1993-11-03T00:00:00-04:56"'
        )
        self.assertEqual(
            serialize.dumps({
                "date": datetime(year=1993, month=11, day=3, tzinfo=ET)
            }),
            '{"date": "1993-11-03T00:00:00-04:56"}'
        )
        self.assertEqual(
            serialize.dumps([
                {"date": datetime(year=1993, month=11, day=3, tzinfo=ET)}
            ]),
            '[{"date": "1993-11-03T00:00:00-04:56"}]'
        )
        self.assertEqual(
            serialize.dumps([
                {"date": [datetime(year=1993, month=11, day=3, tzinfo=ET)]}
            ]),
            '[{"date": ["1993-11-03T00:00:00-04:56"]}]'
        )


class TestISO8601JSONDecoder(TestCase):
    """
    Tests :func:`.serialize.loads` and :class:`.serialize.ISO8601JSONDecoder`.
    """

    def test_decode_with_date(self):
        """Decode JSON with a date value."""
        self.assertEqual(
            {"date": datetime(year=1993, month=11, day=3)},
            serialize.loads('{"date": "1993-11-03"}')
        )
        self.assertEqual(
            [{"date": datetime(year=1993, month=11, day=3)}],
            serialize.loads('[{"date": "1993-11-03"}]')
        )
        self.assertEqual(
            [{"date": [datetime(year=1993, month=11, day=3)]}],
            serialize.loads('[{"date": ["1993-11-03"]}]')
        )
        self.assertEqual(
            [{"date": ["1993"]}],
            serialize.loads('[{"date": ["1993"]}]')
        )

    def test_decode_with_datetime(self):
        """Decode JSON with a datetime value."""
        self.assertEqual(
            {"date": datetime(year=1993, month=11, day=3)},
            serialize.loads('{"date": "1993-11-03T00:00:00"}')
        )
        self.assertEqual(
            [{"date": datetime(year=1993, month=11, day=3)}],
            serialize.loads('[{"date": "1993-11-03T00:00:00"}]')
        )
        self.assertEqual(
            [{"date": [datetime(year=1993, month=11, day=3)]}],
            serialize.loads('[{"date": ["1993-11-03T00:00:00"]}]')
        )

    def test_encode_with_datetime_tz(self):
        """Decode JSON with a datetime value."""
        self.assertEqual(
            {"date": datetime(year=1993, month=11, day=3, tzinfo=ET)},
            serialize.loads('{"date": "1993-11-03T00:00:00-04:56"}')
        )
        self.assertEqual(
            [{"date": datetime(year=1993, month=11, day=3, tzinfo=ET)}],
            serialize.loads('[{"date": "1993-11-03T00:00:00-04:56"}]')
        )
        self.assertEqual(
            [{"date": [datetime(year=1993, month=11, day=3, tzinfo=ET)]}],
            serialize.loads('[{"date": ["1993-11-03T00:00:00-04:56"]}]')
        )



"""Tests for :mod:`arxiv.base.alerts`."""

from unittest import TestCase, mock
from flask import Markup

from arxiv.base import alerts


class TestFlashInfo(TestCase):
    """Tests for :func:`alerts.flash_info`."""

    @mock.patch(f'{alerts.__name__}.flash')
    def test_flash_message(self, mock_flash):
        """Just flash a simple message."""
        alerts.flash_info('The message')
        (data, category), kwargs = mock_flash.call_args
        self.assertEqual(data['message'], 'The message')
        self.assertIsInstance(data['message'], str)
        self.assertTrue(data['dismissable'])
        self.assertEqual(category, alerts.INFO)

    @mock.patch(f'{alerts.__name__}.flash')
    def test_flash_message_no_dismiss(self, mock_flash):
        """Flash a simple message that can't be dismissed."""
        alerts.flash_info('The message', dismissable=False)
        (data, category), kwargs = mock_flash.call_args
        self.assertEqual(data['message'], 'The message')
        self.assertIsInstance(data['message'], str)
        self.assertFalse(data['dismissable'])
        self.assertEqual(category, alerts.INFO)

    @mock.patch(f'{alerts.__name__}.flash')
    def test_safe_flash_message(self, mock_flash):
        """Flash a simple message that is HTML-safe."""
        alerts.flash_info('The message', safe=True)
        (data, category), kwargs = mock_flash.call_args
        self.assertEqual(data['message'], 'The message')
        self.assertIsInstance(data['message'], Markup)
        self.assertTrue(data['dismissable'])
        self.assertEqual(category, alerts.INFO)

    @mock.patch(f'{alerts.__name__}.flash')
    def test_flash_message_with_title(self, mock_flash):
        """Just flash a simple message with a title."""
        alerts.flash_info('The message', title='Foo title')
        (data, category), kwargs = mock_flash.call_args
        self.assertEqual(data['message'], 'The message')
        self.assertEqual(data['title'], 'Foo title')
        self.assertTrue(data['dismissable'])
        self.assertEqual(category, alerts.INFO)
        self.assertIsInstance(data['message'], str)


class TestFlashWarning(TestCase):
    """Tests for :func:`alerts.flash_warning`."""

    @mock.patch(f'{alerts.__name__}.flash')
    def test_flash_message(self, mock_flash):
        """Just flash a simple message."""
        alerts.flash_warning('The message')
        (data, category), kwargs = mock_flash.call_args
        self.assertEqual(data['message'], 'The message')
        self.assertIsInstance(data['message'], str)
        self.assertTrue(data['dismissable'])
        self.assertEqual(category, alerts.WARNING)

    @mock.patch(f'{alerts.__name__}.flash')
    def test_flash_message_no_dismiss(self, mock_flash):
        """Flash a simple message that can't be dismissed."""
        alerts.flash_warning('The message', dismissable=False)
        (data, category), kwargs = mock_flash.call_args
        self.assertEqual(data['message'], 'The message')
        self.assertIsInstance(data['message'], str)
        self.assertFalse(data['dismissable'])
        self.assertEqual(category, alerts.WARNING)

    @mock.patch(f'{alerts.__name__}.flash')
    def test_safe_flash_message(self, mock_flash):
        """Flash a simple message that is HTML-safe."""
        alerts.flash_warning('The message', safe=True)
        (data, category), kwargs = mock_flash.call_args
        self.assertEqual(data['message'], 'The message')
        self.assertIsInstance(data['message'], Markup)
        self.assertTrue(data['dismissable'])
        self.assertEqual(category, alerts.WARNING)

    @mock.patch(f'{alerts.__name__}.flash')
    def test_flash_message_with_title(self, mock_flash):
        """Just flash a simple message with a title."""
        alerts.flash_warning('The message', title='Foo title')
        (data, category), kwargs = mock_flash.call_args
        self.assertEqual(data['message'], 'The message')
        self.assertEqual(data['title'], 'Foo title')
        self.assertTrue(data['dismissable'])
        self.assertEqual(category, alerts.WARNING)
        self.assertIsInstance(data['message'], str)


class TestFlashFailure(TestCase):
    """Tests for :func:`alerts.flash_failure`."""

    @mock.patch(f'{alerts.__name__}.flash')
    def test_flash_message(self, mock_flash):
        """Just flash a simple message."""
        alerts.flash_failure('The message')
        (data, category), kwargs = mock_flash.call_args
        self.assertEqual(data['message'], 'The message')
        self.assertIsInstance(data['message'], str)
        self.assertTrue(data['dismissable'])
        self.assertEqual(category, alerts.FAILURE)

    @mock.patch(f'{alerts.__name__}.flash')
    def test_flash_message_no_dismiss(self, mock_flash):
        """Flash a simple message that can't be dismissed."""
        alerts.flash_failure('The message', dismissable=False)
        (data, category), kwargs = mock_flash.call_args
        self.assertEqual(data['message'], 'The message')
        self.assertIsInstance(data['message'], str)
        self.assertFalse(data['dismissable'])
        self.assertEqual(category, alerts.FAILURE)

    @mock.patch(f'{alerts.__name__}.flash')
    def test_safe_flash_message(self, mock_flash):
        """Flash a simple message that is HTML-safe."""
        alerts.flash_failure('The message', safe=True)
        (data, category), kwargs = mock_flash.call_args
        self.assertEqual(data['message'], 'The message')
        self.assertIsInstance(data['message'], Markup)
        self.assertTrue(data['dismissable'])
        self.assertEqual(category, alerts.FAILURE)

    @mock.patch(f'{alerts.__name__}.flash')
    def test_flash_message_with_title(self, mock_flash):
        """Just flash a simple message with a title."""
        alerts.flash_failure('The message', title='Foo title')
        (data, category), kwargs = mock_flash.call_args
        self.assertEqual(data['message'], 'The message')
        self.assertEqual(data['title'], 'Foo title')
        self.assertTrue(data['dismissable'])
        self.assertEqual(category, alerts.FAILURE)
        self.assertIsInstance(data['message'], str)


class TestFlashSuccess(TestCase):
    """Tests for :func:`alerts.flash_success`."""

    @mock.patch(f'{alerts.__name__}.flash')
    def test_flash_message(self, mock_flash):
        """Just flash a simple message."""
        alerts.flash_success('The message')
        (data, category), kwargs = mock_flash.call_args
        self.assertEqual(data['message'], 'The message')
        self.assertIsInstance(data['message'], str)
        self.assertTrue(data['dismissable'])
        self.assertEqual(category, alerts.SUCCESS)

    @mock.patch(f'{alerts.__name__}.flash')
    def test_flash_message_no_dismiss(self, mock_flash):
        """Flash a simple message that can't be dismissed."""
        alerts.flash_success('The message', dismissable=False)
        (data, category), kwargs = mock_flash.call_args
        self.assertEqual(data['message'], 'The message')
        self.assertIsInstance(data['message'], str)
        self.assertFalse(data['dismissable'])
        self.assertEqual(category, alerts.SUCCESS)

    @mock.patch(f'{alerts.__name__}.flash')
    def test_safe_flash_message(self, mock_flash):
        """Flash a simple message that is HTML-safe."""
        alerts.flash_success('The message', safe=True)
        (data, category), kwargs = mock_flash.call_args
        self.assertEqual(data['message'], 'The message')
        self.assertIsInstance(data['message'], Markup)
        self.assertTrue(data['dismissable'])
        self.assertEqual(category, alerts.SUCCESS)

    @mock.patch(f'{alerts.__name__}.flash')
    def test_flash_message_with_title(self, mock_flash):
        """Just flash a simple message with a title."""
        alerts.flash_success('The message', title='Foo title')
        (data, category), kwargs = mock_flash.call_args
        self.assertEqual(data['message'], 'The message')
        self.assertEqual(data['title'], 'Foo title')
        self.assertTrue(data['dismissable'])
        self.assertEqual(category, alerts.SUCCESS)
        self.assertIsInstance(data['message'], str)
